import { css, html, LitElement, property, svg } from "lit-element";
import { classMap } from "lit-html/directives/class-map";

const BRANCH_HEIGHT = 30;
const BRANCH_CURVATURE = 25;

export class HatGraph extends LitElement {
  @property() _num_items = 0;

  @property({ reflect: true }) branching?;
  @property({ reflect: true }) mark_start?;
  @property({ reflect: true }) mark_end?;
  @property({ reflect: true }) disabled?;

  async updateChildren() {
    this._num_items = this.children.length;
  }

  render() {
    let branches = [];
    let total_width = 0;
    let max_height = 0;
    let min_height = Number.POSITIVE_INFINITY;
    if (this.branching !== undefined) {
      for (const c of Array.from(this.children)) {
        if (c.slot === "head") continue;
        const rect = c.getBoundingClientRect();
        branches.push({
          x: rect.width / 2 + total_width,
          height: rect.height,
          start: c.getAttribute("graph-start") != null,
          end: c.getAttribute("graph-end") != null,
        });
        total_width += rect.width;
        max_height = Math.max(max_height, rect.height);
        min_height = Math.min(min_height, rect.height);
      }
    }

    return html`
      <slot name="head" @slotchange=${this.updateChildren}> </slot>
      ${this.branching !== undefined
        ? svg`
            <svg
              id="top"
              width="${total_width}"
              height="${BRANCH_HEIGHT}"
            >
              ${branches.map((branch, i) => {
                if (branch.start) return "";
                return svg`
                  <path
                    class="${classMap({
                      line: true,
                      marked: this.mark_start === i,
                    })}"
                    id="${this.mark_start === i ? "mark-start" : ""}"
                    index=${i}
                    d="
                      M ${total_width / 2} 0
                      C ${total_width / 2} ${BRANCH_CURVATURE}
                        ${branch.x} ${BRANCH_HEIGHT - BRANCH_CURVATURE}
                        ${branch.x} ${BRANCH_HEIGHT}
                      "/>
                `;
              })}
              <use xlink:href="#mark-start" />
            </svg>
          `
        : ""}
      <div id="branches">
        ${this.branching !== undefined
          ? svg`
              <svg
                id="lines"
                width="${total_width}"
                height="${max_height}"
              >
                ${branches.map((branch, i) => {
                  if (branch.end) return "";
                  return svg`
                    <path
                      class="${classMap({
                        line: true,
                        marked: this.mark_end === i,
                      })}"
                      index=${i}
                      d="
                        M ${branch.x} ${branch.height}
                        l 0 ${max_height - branch.height}
                        "/>
                  `;
                })}
              </svg>
            `
          : ""}
        <slot @slotchange=${this.updateChildren}></slot>
      </div>

      ${this.branching !== undefined
        ? svg`
            <svg
              id="bottom"
              width="${total_width}"
              height="${BRANCH_HEIGHT}"
            >
              ${branches.map((branch, i) => {
                if (branch.end) return "";
                return svg`
                  <path
                    class="${classMap({
                      line: true,
                      marked: this.mark_end === i,
                    })}"
                    id="${this.mark_end === i ? "mark-end" : ""}"
                    index=${i}
                    d="
                      M ${branch.x} 0
                      C ${branch.x} ${BRANCH_CURVATURE}
                        ${total_width / 2} ${BRANCH_HEIGHT - BRANCH_CURVATURE}
                        ${total_width / 2} ${BRANCH_HEIGHT}
                      "/>
                `;
              })}
              <use xlink:href="#mark-end" />
            </svg>
          `
        : ""}
    `;
  }

  static get styles() {
    return css`
      :host {
        position: relative;
        display: flex;
        flex-direction: column;
        align-items: center;
        --stroke-clr: var(--stroke-color, rgb(3, 169, 244));
        --marked-clr: var(--marked-color, green);
        --disabled-clr: var(--disabled-color, gray);
      }
      #branches {
        position: relative;
        display: flex;
        flex-direction: column;
        align-items: center;
      }
      :host([branching]) #branches {
        flex-direction: row;
        align-items: start;
      }

      #lines {
        position: absolute;
        z-index: -1;
      }

      path.line {
        stroke: var(--stroke-clr);
        stroke-width: 2;
        fill: none;
      }
      path.line.marked {
        stroke: var(--marked-clr);
      }
      :host([disabled]) path.line {
        stroke: var(--disabled-clr);
      }
    `;
  }
}

customElements.define("hat-graph", HatGraph);
